from __future__ import (
    absolute_import,
    division,
    print_function,
)

import re

from pcs.test.tools.misc import (
    get_test_resource as rc,
    skip_unless_pacemaker_supports_bundle,
    skip_unless_pacemaker_supports_systemd,
)
from pcs.test.cib_resource.common import ResourceTest


class Success(ResourceTest):
    def test_base_create(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    @skip_unless_pacemaker_supports_systemd()
    def test_base_create_with_agent_name_including_systemd_instance(self):
        # crm_resource returns the same metadata for any systemd resource, no
        # matter if it exists or not
        self.assert_effect(
            "resource create R systemd:test@a:b --no-default-ops",
            """<resources>
                <primitive class="systemd" id="R" type="test@a:b">
                    <operations>
                        <op id="R-monitor-interval-60" interval="60"
                            name="monitor" timeout="100"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_base_create_with_default_ops(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-migrate_from-interval-0s" interval="0s"
                            name="migrate_from" timeout="20"
                        />
                        <op id="R-migrate_to-interval-0s" interval="0s"
                            name="migrate_to" timeout="20"
                        />
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                        <op id="R-reload-interval-0s" interval="0s"
                            name="reload" timeout="20"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="20"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_create_with_options(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:IPaddr2"
                " ip=192.168.0.99 cidr_netmask=32"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="IPaddr2"
                >
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-cidr_netmask"
                            name="cidr_netmask" value="32"
                        />
                        <nvpair id="R-instance_attributes-ip" name="ip"
                            value="192.168.0.99"
                        />
                    </instance_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_create_with_trace_options(self):
        # trace_ra and trace_file options are not defined in metadata but they
        # are allowed for all ocf:heartbeat and ocf:pacemaker agents. This test
        # checks it is possible to set them without --force.
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:Dummy"
                " trace_ra=1 trace_file=/root/trace"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="Dummy"
                >
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-trace_file"
                            name="trace_file" value="/root/trace"
                        />
                        <nvpair id="R-instance_attributes-trace_ra"
                            name="trace_ra" value="1"
                        />
                    </instance_attributes>
                    <operations>
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_create_with_options_and_operations(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:IPaddr2"
                " ip=192.168.0.99 cidr_netmask=32  op monitor interval=30s"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="IPaddr2"
                >
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-cidr_netmask"
                            name="cidr_netmask" value="32"
                        />
                        <nvpair id="R-instance_attributes-ip" name="ip"
                            value="192.168.0.99"
                        />
                    </instance_attributes>
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_create_disabled(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --disabled",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <meta_attributes id="R-meta_attributes">
                        <nvpair id="R-meta_attributes-target-role"
                            name="target-role" value="Stopped"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_with_clone(self):
        self.assert_effect(
            [
                "resource create R ocf:heartbeat:Dummy --no-default-ops --clone"
                ,
                "resource create R ocf:heartbeat:Dummy --no-default-ops clone",
            ],
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                </clone>
            </resources>"""
        )

    def test_with_clone_options(self):
        self.assert_effect(
            [
                "resource create R ocf:heartbeat:Dummy --no-default-ops"
                    " --cloneopt notify=true"
                ,
                "resource create R ocf:heartbeat:Dummy --no-default-ops clone"
                    " notify=true"
                ,
                "resource create R ocf:heartbeat:Dummy --no-default-ops --clone"
                    " notify=true"
                ,
            ],
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-notify"
                            name="notify" value="true"
                        />
                    </meta_attributes>
                </clone>
            </resources>"""
        )

    def test_with_master(self):
        self.assert_effect(
            [
                "resource create R ocf:heartbeat:Dummy --no-default-ops --master",
                "resource create R ocf:heartbeat:Dummy --no-default-ops master",
            ],
            """<resources>
                <master id="R-master">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                </master>
            </resources>"""
        )

    def test_create_with_options_and_meta(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:IPaddr2"
                " ip=192.168.0.99 cidr_netmask=32 meta is-managed=false"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="IPaddr2"
                >
                    <instance_attributes id="R-instance_attributes">
                        <nvpair id="R-instance_attributes-cidr_netmask"
                            name="cidr_netmask" value="32"
                        />
                        <nvpair id="R-instance_attributes-ip" name="ip"
                            value="192.168.0.99"
                        />
                    </instance_attributes>
                    <meta_attributes id="R-meta_attributes">
                        <nvpair id="R-meta_attributes-is-managed"
                            name="is-managed" value="false"
                        />
                    </meta_attributes>
                    <operations>
                        <op id="R-monitor-interval-10s" interval="10s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

class SuccessOperations(ResourceTest):
    def test_create_with_operations(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:Dummy"
                " op monitor interval=30s"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_multiple_op_keyword(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops"
                " op monitor interval=30s op monitor interval=20s"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                        <op id="R-monitor-interval-20s" interval="20s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_multiple_operations_same_op_keyword(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops"
                " op monitor interval=30s monitor interval=20s"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor"
                        />
                        <op id="R-monitor-interval-20s" interval="20s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_multiple_op_options_for_same_action(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops"
                " op monitor interval=30s timeout=20s"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor" timeout="20s"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_op_with_OCF_CHECK_LEVEL(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops"
                " op monitor interval=30s timeout=20s OCF_CHECK_LEVEL=1"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-30s" interval="30s"
                            name="monitor" timeout="20s"
                        >
                            <instance_attributes
                                id="R-monitor-interval-30s-instance_attributes"
                            >
                                <nvpair
                                    id="R-monitor-interval-30s-"""
                                        +'instance_attributes-OCF_CHECK_LEVEL"'
                                        +"""
                                    name="OCF_CHECK_LEVEL" value="1"
                                />
                            </instance_attributes>
                        </op>
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_default_ops_only(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-migrate_from-interval-0s" interval="0s"
                            name="migrate_from" timeout="20"
                        />
                        <op id="R-migrate_to-interval-0s" interval="0s"
                            name="migrate_to" timeout="20"
                        />
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                        <op id="R-reload-interval-0s" interval="0s"
                            name="reload" timeout="20"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="20"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_merging_default_ops_explictly_specified(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy op start timeout=200",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-migrate_from-interval-0s" interval="0s"
                            name="migrate_from" timeout="20"
                        />
                        <op id="R-migrate_to-interval-0s" interval="0s"
                            name="migrate_to" timeout="20"
                        />
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                        <op id="R-reload-interval-0s" interval="0s"
                            name="reload" timeout="20"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="200"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_completing_monitor_operation(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:Dummy",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )

    def test_adapt_second_op_interval(self):
        self.assert_effect(
            "resource create R ocf:pacemaker:Stateful",
            """<resources>
                <primitive class="ocf" id="R" provider="pacemaker"
                    type="Stateful"
                >
                    <operations>
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" role="Master" timeout="20"
                        />
                        <op id="R-monitor-interval-11" interval="11"
                            name="monitor" role="Slave" timeout="20"
                        />
                        <op id="R-notify-interval-0s" interval="0s"
                            name="notify" timeout="5"
                        />
                        <op id="R-start-interval-0s" interval="0s" name="start"
                            timeout="20"
                        />
                        <op id="R-stop-interval-0s" interval="0s" name="stop"
                            timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
            ,
            "Warning: changing a monitor operation interval from 10 to 11 to"
                " make the operation unique\n"
        )

    def test_warn_on_forced_unknown_operation(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:Dummy"
                " op monitro interval=30s --force"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="R-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                        <op id="R-monitro-interval-30s" interval="30s"
                            name="monitro"
                        />
                    </operations>
                </primitive>
            </resources>"""
            ,
            "Warning: 'monitro' is not a valid operation name value, use"
                " meta-data, migrate_from, migrate_to, monitor, reload, start,"
                " stop, validate-all\n"
        )

    def test_op_id(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:Dummy"
                " op monitor interval=30s id=abcd"
            ,
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat" type="Dummy">
                    <operations>
                        <op id="abcd" interval="30s" name="monitor" />
                    </operations>
                </primitive>
            </resources>"""
        )

class SuccessGroup(ResourceTest):
    def test_with_group(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --group G",
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>"""
        )

    def test_with_existing_group(self):
        self.assert_pcs_success(
            "resource create R0 ocf:heartbeat:Dummy --no-default-ops --group G"
        )
        self.assert_effect(
            [
                "resource create R ocf:heartbeat:Dummy --no-default-ops --group"
                    " G"
                ,
                "resource create R ocf:heartbeat:Dummy --no-default-ops --group"
                    " G --after R0"
                ,
            ],
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R0" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>"""
        )

    def test_with_group_with_after(self):
        self.assert_pcs_success_all([
            "resource create R0 ocf:heartbeat:Dummy --no-default-ops --group G",
            "resource create R1 ocf:heartbeat:Dummy --no-default-ops --group G",
        ])
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --group G"
                " --after R0"
            ,
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R0" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R1" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R1-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>

                </group>
            </resources>"""
        )

    def test_with_group_with_before(self):
        self.assert_pcs_success(
            "resource create R0 ocf:heartbeat:Dummy --no-default-ops --group G"
        )
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --group G"
                " --before R0"
            ,
            """<resources>
                <group id="G">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <primitive class="ocf" id="R0" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R0-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                </group>
            </resources>"""
        )

class SuccessMaster(ResourceTest):
    def test_disable_is_on_master_element(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --disabled --master",
            """<resources>
                <master id="R-master">
                    <meta_attributes id="R-master-meta_attributes">
                        <nvpair id="R-master-meta_attributes-target-role"
                            name="target-role" value="Stopped"
                        />
                    </meta_attributes>
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                </master>
            </resources>"""
        )

    def test_put_options_after_master_as_its_meta_fix_1(self):
        """
        fixes bz 1378107 (do not use master options as primitive options)
        """
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy state=a"
                " --master is-managed=false --force"
            ,
            """<resources>
                <master id="R-master">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <instance_attributes id="R-instance_attributes">
                            <nvpair id="R-instance_attributes-state"
                                name="state" value="a"
                            />
                        </instance_attributes>
                        <operations>
                            <op id="R-migrate_from-interval-0s" interval="0s"
                                name="migrate_from" timeout="20"
                            />
                            <op id="R-migrate_to-interval-0s" interval="0s"
                                name="migrate_to" timeout="20"
                            />
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                            <op id="R-reload-interval-0s" interval="0s"
                                name="reload" timeout="20"
                            />
                            <op id="R-start-interval-0s" interval="0s"
                                name="start" timeout="20"
                            />
                            <op id="R-stop-interval-0s" interval="0s"
                                name="stop" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-master-meta_attributes">
                        <nvpair id="R-master-meta_attributes-is-managed"
                            name="is-managed" value="false"
                    />
                    </meta_attributes>
                </master>
            </resources>"""
        )

    def test_put_options_after_master_as_its_meta_fix_2(self):
        """
        fixes bz 1378107 (do not use master options as operations)
        """
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy state=a op monitor"
                " interval=10s --master is-managed=false --force"
                " --no-default-ops"
            ,
            """<resources>
                <master id="R-master">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <instance_attributes id="R-instance_attributes">
                            <nvpair id="R-instance_attributes-state"
                                name="state" value="a"
                            />
                        </instance_attributes>
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-master-meta_attributes">
                        <nvpair id="R-master-meta_attributes-is-managed"
                            name="is-managed" value="false"
                    />
                    </meta_attributes>
                </master>
            </resources>"""
        )

    def test_do_not_steal_primitive_meta_options(self):
        """
        fixes bz 1378107
        """
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy meta a=b --master b=c"
                " --no-default-ops"
            ,
            """<resources>
                <master id="R-master">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <meta_attributes id="R-meta_attributes">
                            <nvpair id="R-meta_attributes-a" name="a"
                                value="b"
                            />
                        </meta_attributes>
                        <operations>
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-master-meta_attributes">
                        <nvpair id="R-master-meta_attributes-b" name="b"
                            value="c"
                        />
                    </meta_attributes>
                </master>
            </resources>"""
        )

    def test_takes_master_meta_attributes(self):
        self.assert_effect(
            "resource create --no-default-ops R ocf:heartbeat:IPaddr2"
                " ip=192.168.0.99 --master cidr_netmask=32"
            ,
            """<resources>
                <master id="R-master">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="IPaddr2"
                    >
                        <instance_attributes id="R-instance_attributes">
                            <nvpair id="R-instance_attributes-ip" name="ip"
                                value="192.168.0.99"
                            />
                        </instance_attributes>
                        <operations>
                            <op id="R-monitor-interval-10s" interval="10s"
                                name="monitor" timeout="20s"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-master-meta_attributes">
                        <nvpair id="R-master-meta_attributes-cidr_netmask"
                            name="cidr_netmask" value="32"
                        />
                    </meta_attributes>
                </master>
            </resources>"""
        )

class SuccessClone(ResourceTest):
    def test_clone_does_not_overshadow_meta_options(self):
        self.assert_effect(
            [
                "resource create R ocf:heartbeat:Dummy meta a=b --clone c=d",
                "resource create R ocf:heartbeat:Dummy --clone c=d meta a=b",
            ],
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <meta_attributes id="R-meta_attributes">
                            <nvpair id="R-meta_attributes-a" name="a"
                                value="b"
                            />
                        </meta_attributes>
                        <operations>
                            <op id="R-migrate_from-interval-0s" interval="0s"
                                name="migrate_from" timeout="20"
                            />
                            <op id="R-migrate_to-interval-0s" interval="0s"
                                name="migrate_to" timeout="20"
                            />
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                            <op id="R-reload-interval-0s" interval="0s"
                                name="reload" timeout="20"
                            />
                            <op id="R-start-interval-0s" interval="0s"
                                name="start" timeout="20"
                            />
                            <op id="R-stop-interval-0s" interval="0s"
                                name="stop" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-c" name="c"
                            value="d"
                        />
                    </meta_attributes>
                </clone>
            </resources>"""
        )

    def test_clone_does_not_overshadow_operations(self):
        self.assert_effect(
            [
                "resource create R ocf:heartbeat:Dummy op monitor interval=10"
                    " --clone c=d"
                ,
                "resource create R ocf:heartbeat:Dummy --clone c=d"
                    " op monitor interval=10"
                ,
            ],
            """<resources>
                <clone id="R-clone">
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-migrate_from-interval-0s" interval="0s"
                                name="migrate_from" timeout="20"
                            />
                            <op id="R-migrate_to-interval-0s" interval="0s"
                                name="migrate_to" timeout="20"
                            />
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor"
                            />
                            <op id="R-reload-interval-0s" interval="0s"
                                name="reload" timeout="20"
                            />
                            <op id="R-start-interval-0s" interval="0s"
                                name="start" timeout="20"
                            />
                            <op id="R-stop-interval-0s" interval="0s"
                                name="stop" timeout="20"
                            />
                        </operations>
                    </primitive>
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-c" name="c"
                            value="d"
                        />
                    </meta_attributes>
                </clone>
            </resources>"""
        )

    def test_clone_places_disabled_correctly(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:Dummy --clone --disabled",
            """<resources>
                <clone id="R-clone">
                    <meta_attributes id="R-clone-meta_attributes">
                        <nvpair id="R-clone-meta_attributes-target-role"
                            name="target-role" value="Stopped"
                        />
                    </meta_attributes>
                    <primitive class="ocf" id="R" provider="heartbeat"
                        type="Dummy"
                    >
                        <operations>
                            <op id="R-migrate_from-interval-0s" interval="0s"
                                name="migrate_from" timeout="20"
                            />
                            <op id="R-migrate_to-interval-0s" interval="0s"
                                name="migrate_to" timeout="20"
                            />
                            <op id="R-monitor-interval-10" interval="10"
                                name="monitor" timeout="20"
                            />
                            <op id="R-reload-interval-0s" interval="0s"
                                name="reload" timeout="20"
                            />
                            <op id="R-start-interval-0s" interval="0s"
                                name="start" timeout="20"
                            />
                            <op id="R-stop-interval-0s" interval="0s"
                                name="stop" timeout="20"
                            />
                        </operations>
                    </primitive>
                </clone>
            </resources>"""
        )


@skip_unless_pacemaker_supports_bundle
class Bundle(ResourceTest):
    empty_cib = rc("cib-empty-2.8.xml")

    def fixture_primitive(self, name, bundle=None):
        if bundle:
            self.assert_pcs_success(
                "resource create {0} ocf:heartbeat:Dummy bundle {1}".format(
                    name, bundle
                )
            )
        else:
            self.assert_pcs_success(
                "resource create {0} ocf:heartbeat:Dummy".format(name)
            )

    def fixture_bundle(self, name):
        self.assert_pcs_success(
            (
                "resource bundle create {0} container docker image=pcs:test "
                "network control-port=1234"
            ).format(name)
        )

    def test_bundle_id_not_specified(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops bundle"
            ,
            "Error: you have to specify exactly one bundle\n"
        )

    def test_bundle_id_is_not_bundle(self):
        self.fixture_primitive("R1")
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy bundle R1",
            "Error: 'R1' is not a bundle\n"
        )

    def test_bundle_id_does_not_exist(self):
        self.assert_pcs_fail(
            "resource create R1 ocf:heartbeat:Dummy bundle B",
            "Error: bundle 'B' does not exist\n"
        )

    def test_primitive_already_in_bundle(self):
        self.fixture_bundle("B")
        self.fixture_primitive("R1", bundle="B")
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy --no-default-ops bundle B",
            (
                "Error: bundle 'B' already contains resource 'R1', a bundle "
                "may contain at most one resource\n"
            )
        )

    def test_success(self):
        self.fixture_bundle("B")
        self.assert_effect(
            "resource create R1 ocf:heartbeat:Dummy --no-default-ops bundle B",
            """
                <resources>
                    <bundle id="B">
                        <docker image="pcs:test" />
                        <network control-port="1234"/>
                        <primitive class="ocf" id="R1" provider="heartbeat"
                            type="Dummy"
                        >
                            <operations>
                                <op id="R1-monitor-interval-10" interval="10"
                                    name="monitor" timeout="20"
                                />
                            </operations>
                        </primitive>
                    </bundle>
                </resources>
            """
        )


class FailOrWarn(ResourceTest):
    def test_error_group_clone_combination(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --clone"
                " --group G"
            ,
            "Error: you can specify only one of clone, master, bundle or"
                " --group\n"
        )

    def test_error_master_clone_combination(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --clone"
                " --master"
            ,
            "Error: you can specify only one of clone, master, bundle or"
                " --group\n"
        )

    def test_error_master_group_combination(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --master"
                " --group G"
            ,
            "Error: you can specify only one of clone, master, bundle or"
                " --group\n"
        )

    def test_error_bundle_clone_combination(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --clone"
                " bundle bundle_id"
            ,
            "Error: you can specify only one of clone, master, bundle or"
                " --group\n"
        )

    def test_error_bundle_master_combination(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --master"
                " bundle bundle_id"
            ,
            "Error: you can specify only one of clone, master, bundle or"
                " --group\n"
        )

    def test_error_bundle_group_combination(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --no-default-ops --group G"
                " bundle bundle_id"
            ,
            "Error: you can specify only one of clone, master, bundle or"
                " --group\n"
        )

    def test_fail_when_nonexisting_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:NoExisting",
            # pacemaker 1.1.18 changes -5 to Input/output error
            stdout_regexp=re.compile("^"
                "Error: Agent 'ocf:heartbeat:NoExisting' is not installed or "
                "does not provide valid metadata: Metadata query for "
                "ocf:heartbeat:NoExisting failed: (-5|Input/output error), use "
                "--force to override\n"
                "$", re.MULTILINE
            )
        )

    def test_warn_when_forcing_noexistent_agent(self):
        self.assert_effect(
            "resource create R ocf:heartbeat:NoExisting --force",
            """<resources>
                <primitive class="ocf" id="R" provider="heartbeat"
                    type="NoExisting"
                >
                    <operations>
                        <op id="R-monitor-interval-60s" interval="60s"
                            name="monitor"
                        />
                    </operations>
                </primitive>
            </resources>""",
            # pacemaker 1.1.18 changes -5 to Input/output error
            output_regexp=re.compile("^"
                "Warning: Agent 'ocf:heartbeat:NoExisting' is not installed or "
                    "does not provide valid metadata: Metadata query for "
                    "ocf:heartbeat:NoExisting failed: (-5|Input/output error)\n"
                "$", re.MULTILINE
            )
        )


    def test_fail_on_invalid_resource_agent_name(self):
        self.assert_pcs_fail(
            "resource create R invalid_agent_name",
            "Error: Unable to find agent 'invalid_agent_name', try specifying"
                " its full name\n"
        )

    def test_fail_on_invalid_resource_agent_name_even_if_forced(self):
        self.assert_pcs_fail(
            "resource create R invalid_agent_name --force",
            "Error: Unable to find agent 'invalid_agent_name', try specifying"
                " its full name\n"
        )

    def test_fail_when_invalid_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat: --force",
            "Error: Invalid resource agent name 'ocf:heartbeat:'. Use"
                " standard:provider:type when standard is 'ocf' or"
                " standard:type otherwise. List of standards and providers can"
                " be obtained by using commands 'pcs resource standards' and"
                " 'pcs resource providers'\n"
        )

    def test_vail_when_agent_class_is_not_allowed(self):
        self.assert_pcs_fail(
            "resource create R invalid:Dummy --force",
            "Error: Invalid resource agent name 'invalid:Dummy'. Use"
                " standard:provider:type when standard is 'ocf' or"
                " standard:type otherwise. List of standards and providers can"
                " be obtained by using commands 'pcs resource standards' and"
                " 'pcs resource providers'\n"
        )

    def test_fail_when_missing_provider_with_ocf_resource_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:Dummy",
            "Error: Invalid resource agent name 'ocf:Dummy'. Use"
                " standard:provider:type when standard is 'ocf' or"
                " standard:type otherwise. List of standards and providers can"
                " be obtained by using commands 'pcs resource standards' and"
                " 'pcs resource providers'\n"
        )

    def test_fail_when_provider_appear_with_non_ocf_resource_agent(self):
        self.assert_pcs_fail(
            "resource create R lsb:provider:Dummy --force",
            "Error: Invalid resource agent name 'lsb:provider:Dummy'. Use"
                " standard:provider:type when standard is 'ocf' or"
                " standard:type otherwise. List of standards and providers can"
                " be obtained by using commands 'pcs resource standards' and"
                " 'pcs resource providers'\n"
        )

    def test_print_info_about_agent_completion(self):
        self.assert_pcs_success(
            "resource create R delay",
            "Assumed agent name 'ocf:heartbeat:Delay' (deduced from 'delay')\n"
        )

    def test_fail_for_unambiguous_agent(self):
        self.assert_pcs_fail(
            "resource create R Dummy",
            "Error: Multiple agents match 'Dummy', please specify full name:"
                " ocf:heartbeat:Dummy, ocf:pacemaker:Dummy\n"
        )

    def test_for_options_not_matching_resource_agent(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy a=b c=d",
            "Error: invalid resource options: 'a', 'c', allowed options are: "
                "fake, state, trace_file, trace_ra, use --force to override\n"
        )

    def test_for_missing_options_of_resource_agent(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R IPaddr2",
            "Error: required resource option 'ip' is missing,"
                " use --force to override\n"
                "Assumed agent name 'ocf:heartbeat:IPaddr2' (deduced from"
                " 'IPaddr2')\n"
        )

    def test_fail_on_invalid_resource_id(self):
        self.assert_pcs_fail(
            "resource create #R ocf:heartbeat:Dummy",
            "Error: invalid resource name '#R',"
                " '#' is not a valid first character for a resource name\n"
        )

    def test_fail_on_existing_resource_id(self):
        self.assert_pcs_success("resource create R ocf:heartbeat:Dummy")
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy",
            "Error: 'R' already exists\n"
        )

    def test_fail_on_invalid_operation_id(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy op monitor interval=30 id=#O",
            "Error: invalid operation id '#O',"
                " '#' is not a valid first character for a operation id\n"
        )

    def test_fail_on_existing_operation_id(self):
        self.assert_pcs_success("resource create R ocf:heartbeat:Dummy")
        self.assert_pcs_fail(
            "resource create S ocf:heartbeat:Dummy op monitor interval=30 id=R",
            "Error: 'R' already exists\n"
        )

    def test_fail_on_duplicate_operation_id(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy"
                " op monitor interval=30 id=O"
                " op monitor interval=60 id=O"
            ,
            "Error: 'O' already exists\n"
        )

    def test_fail_on_resource_id_same_as_operation_id(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy op monitor interval=30 id=R",
            "Error: 'R' already exists\n"
        )

    def test_fail_on_unknown_operation(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy op monitro interval=100",
            "Error: 'monitro' is not a valid operation name value, use"
                " meta-data, migrate_from, migrate_to, monitor, reload, start,"
                " stop, validate-all, use --force to override\n"
        )

    def test_fail_on_ambiguous_value_of_option(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy op monitor timeout=10"
                " timeout=20"
            ,
            "Error: duplicate option 'timeout' with different values '10' and"
                " '20'\n"
        )

class FailOrWarnOp(ResourceTest):
    def test_fail_empty(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy"
                " op meta is-managed=false"
            ,
            "Error: When using 'op' you must specify an operation name and at"
                " least one option\n"
        )

    def test_fail_only_name_without_any_option(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy"
                " op monitor meta is-managed=false"
            ,
            "Error: When using 'op' you must specify an operation name and at"
                " least one option\n"
        )

    def test_fail_duplicit(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor interval=1h monitor interval=3600sec"
                " monitor interval=1min monitor interval=60s"
            ,
            [
                "Error: multiple specification of the same operation with the"
                    " same interval:"
                ,
                "monitor with intervals 1h, 3600sec",
                "monitor with intervals 1min, 60s",
            ]
        )

    def test_fail_invalid_first_action(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " mo=nitor interval=1min"
            ,
            "Error: When using 'op' you must specify an operation name after"
                " 'op'\n"
            ,
        )

    def test_fail_invalid_option(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor interval=1min moni=tor timeout=80s"
            ,
            "Error: invalid resource operation option 'moni', allowed options"
                " are: OCF_CHECK_LEVEL, description, enabled, id, interval,"
                " interval-origin, name, on-fail, record-pending, requires,"
                " role, start-delay, timeout\n"
        )

    def test_fail_on_invalid_role(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor role=abc"
            ,
            "Error: 'abc' is not a valid role value, use Master, Slave,"
                " Started, Stopped\n"
        )

    def test_force_invalid_role(self):
        self.assert_pcs_fail(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor role=abc --force"
            ,
            "Error: 'abc' is not a valid role value, use Master, Slave,"
                " Started, Stopped\n"
        )

    def test_fail_on_invalid_requires(self):
        self.assert_pcs_fail_regardless_of_force(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor requires=Abc"
            ,
            "Error: 'Abc' is not a valid requires value, use fencing, nothing,"
                " quorum, unfencing\n"
        )

    def test_fail_on_invalid_on_fail(self):
        self.assert_pcs_fail_regardless_of_force(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor on-fail=Abc"
            ,
            "Error: 'Abc' is not a valid on-fail value, use block, fence,"
                " ignore, restart, restart-container, standby, stop\n"
        )

    def test_fail_on_invalid_record_pending(self):
        self.assert_pcs_fail_regardless_of_force(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor record-pending=Abc"
            ,
            "Error: 'Abc' is not a valid record-pending value, use 0, 1, false,"
                " true\n"
        )

    def test_fail_on_invalid_enabled(self):
        self.assert_pcs_fail_regardless_of_force(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor enabled=Abc"
            ,
            "Error: 'Abc' is not a valid enabled value, use 0, 1, false, true\n"
        )

    def test_fail_on_combination_of_start_delay_and_interval_origin(self):
        self.assert_pcs_fail_regardless_of_force(
            "resource create --no-default-ops R ocf:heartbeat:Dummy op"
                " monitor start-delay=10 interval-origin=20"
            ,
            "Error: Only one of resource operation options 'interval-origin'"
                " and 'start-delay' can be used\n"
        )

class FailOrWarnGroup(ResourceTest):
    def test_fail_when_invalid_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --group 1",
            "Error: invalid group name '1', '1' is not a valid first character"
                " for a group name\n"
        )

    def test_fail_when_try_use_id_of_another_element(self):
        self.assert_effect(
            "resource create R1 ocf:heartbeat:Dummy --no-default-ops meta a=b",
            """<resources>
                <primitive class="ocf" id="R1" provider="heartbeat"
                    type="Dummy"
                >
                    <meta_attributes id="R1-meta_attributes">
                        <nvpair id="R1-meta_attributes-a" name="a" value="b"/>
                    </meta_attributes>
                    <operations>
                        <op id="R1-monitor-interval-10" interval="10"
                            name="monitor" timeout="20"
                        />
                    </operations>
                </primitive>
            </resources>"""
        )
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy --group R1-meta_attributes",
            "Error: 'R1-meta_attributes' is not a group\n"
        )


    def test_fail_when_entered_both_after_and_before(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --group G --after S1 --before S2",
            "Error: you cannot specify both --before and --after\n"
        )

    def test_fail_when_after_is_used_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --after S1",
            "Error: you cannot use --after without --group\n"
        )

    def test_fail_when_before_is_used_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --before S1",
            "Error: you cannot use --before without --group\n"
        )

    def test_fail_when_before_after_conflicts_and_moreover_without_group(self):
        self.assert_pcs_fail(
            "resource create R ocf:heartbeat:Dummy --after S1 --before S2",
            "Error: you cannot specify both --before and --after"
                " and you have to specify --group\n"
        )

    def test_fail_when_before_does_not_exist(self):
        self.assert_pcs_success(
            "resource create R0 ocf:heartbeat:Dummy --group G1 "
        )
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy --group G1 --before R1",
            "Error: there is no resource 'R1' in the group 'G1'\n"
        )

    def test_fail_when_use_before_with_new_group(self):
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy --group G1 --before R1",
            "Error: there is no resource 'R1' in the group 'G1'\n"
        )

    def test_fail_when_after_does_not_exist(self):
        self.assert_pcs_success(
            "resource create R0 ocf:heartbeat:Dummy --group G1 "
        )
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy --group G1 --after R1",
            "Error: there is no resource 'R1' in the group 'G1'\n"
        )

    def test_fail_when_use_after_with_new_group(self):
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy --group G1 --after R1",
            "Error: there is no resource 'R1' in the group 'G1'\n"
        )

    def test_fail_when_on_pacemaker_remote_attempt(self):
        self.assert_pcs_fail(
            "resource create R2 ocf:pacemaker:remote",
            "Error: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'"
                ", use --force to override\n"
        )

    def test_warn_when_on_pacemaker_remote_attempt(self):
        self.assert_pcs_success(
            "resource create R2 ocf:pacemaker:remote --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

    def test_fail_when_on_pacemaker_remote_conflict_with_existing_node(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:pacemaker:remote server=R --force",
            "Error: 'R' already exists\n"
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

    def test_fail_when_on_pacemaker_remote_conflict_with_existing_id(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=R2 --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:pacemaker:remote --force",
            "Error: 'R2' already exists\n"
                "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

    def test_fail_when_on_guest_conflict_with_existing_node(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy meta remote-node=R --force",
            "Error: 'R' already exists\n"
                "Warning: this command is not sufficient for creating a guest node"
                ", use 'pcs cluster node add-guest'\n"
        )

    def test_fail_when_on_guest_conflict_with_existing_node_host(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=HOST --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy meta remote-node=HOST"
                " --force"
            ,
            "Error: 'HOST' already exists\n"
                "Warning: this command is not sufficient for creating a guest node"
                ", use 'pcs cluster node add-guest'\n"
        )

    def test_fail_when_on_guest_conflict_with_existing_node_host_addr(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=HOST --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy meta remote-node=A"
                " remote-addr=HOST --force"
            ,
            "Error: 'HOST' already exists\n"
                "Warning: this command is not sufficient for creating a guest node"
                ", use 'pcs cluster node add-guest'\n"
        )

    def test_not_fail_when_on_guest_when_conflict_host_with_name(self):
        self.assert_pcs_success(
            "resource create R ocf:pacemaker:remote server=HOST --force",
            "Warning: this command is not sufficient for creating a remote"
                " connection, use 'pcs cluster node add-remote'\n"
        )

        self.assert_pcs_success(
            "resource create R2 ocf:heartbeat:Dummy meta remote-node=HOST"
                " remote-addr=R --force"
            ,
            "Warning: this command is not sufficient for creating a guest node, use"
                " 'pcs cluster node add-guest'\n"
        )

    def test_fail_when_on_pacemaker_remote_guest_attempt(self):
        self.assert_pcs_fail(
            "resource create R2 ocf:heartbeat:Dummy meta remote-node=HOST",
            "Error: this command is not sufficient for creating a guest node,"
            " use 'pcs cluster node add-guest', use --force to override\n"
        )

    def test_warn_when_on_pacemaker_remote_guest_attempt(self):
        self.assert_pcs_success(
            "resource create R2 ocf:heartbeat:Dummy meta remote-node=HOST"
                " --force"
            ,
            "Warning: this command is not sufficient for creating a guest node,"
            " use 'pcs cluster node add-guest'\n"
        )



from __future__ import (
    absolute_import,
    division,
    print_function,
)

from pcs.test.tools.pcs_unittest import TestCase
from pcs.cli.cluster import command

class ParseNodeAddRemote(TestCase):
    def test_deal_with_explicit_name(self):
        self.assertEqual(
            command._node_add_remote_separate_host_and_name(
                ["host", "name", "a=b"]
            ),
            ("host", "name", ["a=b"])
        )

    def test_deal_with_implicit_name(self):
        self.assertEqual(
            command._node_add_remote_separate_host_and_name(["host", "a=b"]),
            ("host", "host", ["a=b"])
        )



from __future__ import (
    absolute_import,
    division,
    print_function,
)

import re

from pcs.lib.booth import config_structure, reports
from pcs.lib.errors import LibraryError


class InvalidLines(Exception):
    pass

def parse(content):
    try:
        return organize_lines(parse_to_raw_lines(content))
    except InvalidLines as e:
        raise LibraryError(
            reports.booth_config_unexpected_lines(e.args[0])
        )

def build(config_line_list):
    newline = [""]
    return "\n".join(build_to_lines(config_line_list) + newline)

def build_to_lines(config_line_list, deep=0):
    line_list = []
    for key, value, details in config_line_list:
        line_value = value if key != "ticket" else '"{0}"'.format(value)
        line_list.append("{0}{1} = {2}".format("  "*deep, key, line_value))
        if details:
            line_list.extend(build_to_lines(details, deep+1))
    return line_list


def organize_lines(raw_line_list):
    #Decision: Global key is moved up when is below ticket. Alternative is move
    #it below all ticket details. But it is confusing.
    global_section = []
    ticket_section = []
    current_ticket = None
    for key, value in raw_line_list:
        if key == "ticket":
            current_ticket = config_structure.ConfigItem(key, value)
            ticket_section.append(current_ticket)
        elif key in config_structure.GLOBAL_KEYS or not current_ticket:
            global_section.append(config_structure.ConfigItem(key, value))
        else:
            current_ticket.details.append(
                config_structure.ConfigItem(key, value)
            )

    return global_section + ticket_section

def search_with_multiple_re(re_object_list, string):
    """
    return MatchObject of first matching regular expression object or None
    list re_object_list contains regular expresssion objects (products of
        re.compile)
    """
    for expression in re_object_list:
        match = expression.search(string)
        if match:
            return match
    return None

def parse_to_raw_lines(config_content):
    keyword_part = r"^(?P<key>[a-zA-Z0-9_-]+)\s*=\s*"
    expression_list = [re.compile(pattern.format(keyword_part)) for pattern in [
        r"""{0}(?P<value>[^'"]+)$""",
        r"""{0}'(?P<value>[^']*)'\s*(#.*)?$""",
        r"""{0}"(?P<value>[^"]*)"\s*(#.*)?$""",
    ]]

    line_list = []
    invalid_line_list = []
    for line in config_content.splitlines():
        line = line.strip()
        match = search_with_multiple_re(expression_list, line)
        if match:
            line_list.append((match.group("key"), match.group("value")))
        elif line and not line.startswith("#"):
            invalid_line_list.append(line)

    if invalid_line_list:
        raise InvalidLines(invalid_line_list)

    return line_list



from __future__ import (
    absolute_import,
    division,
    print_function,
)

from pcs.common import report_codes as codes

def format_booth_default(value, template):
    return "" if value in ("booth", "", None) else template.format(value)

#Each value (a callable taking report_item.info) returns a message.
#Force text will be appended if necessary.
#If it is necessary to put the force text inside the string then the callable
#must take the force_text parameter.
CODE_TO_MESSAGE_BUILDER_MAP = {
    codes.BOOTH_LACK_OF_SITES: lambda info:
        "lack of sites for booth configuration (need 2 at least): sites {0}"
        .format(", ".join(info["sites"]) if info["sites"] else "missing")
    ,

    codes.BOOTH_EVEN_PEERS_NUM: lambda info:
        "odd number of peers is required (entered {number} peers)"
        .format(**info)
    ,

    codes.BOOTH_ADDRESS_DUPLICATION: lambda info:
        "duplicate address for booth configuration: {0}"
        .format(", ".join(info["addresses"]))
    ,

    codes.BOOTH_CONFIG_UNEXPECTED_LINES: lambda info:
        "unexpected line appeard in config: \n{0}"
        .format("\n".join(info["line_list"]))
    ,

    codes.BOOTH_INVALID_NAME: lambda info:
        "booth name '{name}' is not valid ({reason})"
        .format(**info)
    ,

    codes.BOOTH_TICKET_NAME_INVALID: lambda info:
        "booth ticket name '{0}' is not valid, use alphanumeric chars or dash"
        .format(info["ticket_name"])
    ,

    codes.BOOTH_TICKET_DUPLICATE: lambda info:
        "booth ticket name '{ticket_name}' already exists in configuration"
        .format(**info)
    ,

    codes.BOOTH_TICKET_DOES_NOT_EXIST: lambda info:
        "booth ticket name '{ticket_name}' does not exist"
        .format(**info)
    ,

    codes.BOOTH_ALREADY_IN_CIB: lambda info:
        "booth instance '{name}' is already created as cluster resource"
        .format(**info)
    ,

    codes.BOOTH_NOT_EXISTS_IN_CIB: lambda info:
        "booth instance '{name}' not found in cib"
        .format(**info)
    ,

    codes.BOOTH_CONFIG_IS_USED: lambda info:
        "booth instance '{0}' is used{1}".format(
            info["name"],
            " {0}".format(info["detail"]) if info["detail"] else "",
        )
    ,

    codes.BOOTH_MULTIPLE_TIMES_IN_CIB: lambda info:
        "found more than one booth instance '{name}' in cib"
        .format(**info)
    ,

    codes.BOOTH_CONFIG_DISTRIBUTION_STARTED: lambda info:
        "Sending booth configuration to cluster nodes..."
    ,

    codes.BOOTH_CONFIG_ACCEPTED_BY_NODE: lambda info:
        "{node_info}Booth config{desc} saved.".format(
            desc=(
                "" if info["name_list"] in [None, [], ["booth"]]
                else "(s) ({0})".format(", ".join(info["name_list"]))
            ),
            node_info="{0}: ".format(info["node"]) if info["node"] else ""
        )
    ,

    codes.BOOTH_CONFIG_DISTRIBUTION_NODE_ERROR: lambda info:
        "Unable to save booth config{desc} on node '{node}': {reason}".format(
            desc=format_booth_default(info["name"], " ({0})"),
            **info
        )
    ,

    codes.BOOTH_CONFIG_READ_ERROR: lambda info:
        "Unable to read booth config{desc}.".format(
            desc=format_booth_default(info["name"], " ({0})")
        )
    ,

    codes.BOOTH_FETCHING_CONFIG_FROM_NODE: lambda info:
        "Fetching booth config{desc} from node '{node}'...".format(
            desc=format_booth_default(info["config"], " '{0}'"),
            **info
        )
    ,

    codes.BOOTH_DAEMON_STATUS_ERROR: lambda info:
        "unable to get status of booth daemon: {reason}".format(**info)
    ,

    codes.BOOTH_TICKET_STATUS_ERROR: "unable to get status of booth tickets",

    codes.BOOTH_PEERS_STATUS_ERROR: "unable to get status of booth peers",

    codes.BOOTH_CANNOT_DETERMINE_LOCAL_SITE_IP: lambda info:
        "cannot determine local site ip, please specify site parameter"
    ,

    codes.BOOTH_TICKET_OPERATION_FAILED: lambda info:
        (
            "unable to {operation} booth ticket '{ticket_name}'"
            " for site '{site_ip}', reason: {reason}"
        ).format(**info)

    ,

    codes.BOOTH_SKIPPING_CONFIG: lambda info:
        "Skipping config file '{config_file}': {reason}".format(**info)
    ,

    codes.BOOTH_CANNOT_IDENTIFY_KEYFILE:
        "cannot identify authfile in booth configuration"
    ,
}



from __future__ import (
    absolute_import,
    division,
    print_function,
)

from lxml import etree

from pcs.common import report_codes
from pcs.lib.cib.resource import guest_node
from pcs.lib.errors import ReportItemSeverity as severities
from pcs.test.tools.assertions import(
    assert_xml_equal,
    assert_report_item_list_equal,
)
from pcs.test.tools.misc import create_setup_patch_mixin
from pcs.test.tools.pcs_unittest import TestCase
from pcs.lib.node import NodeAddresses


SetupPatchMixin = create_setup_patch_mixin(guest_node)

class ValidateHostConflicts(TestCase):
    def validate(self, node_name, options):
        tree = etree.fromstring("""
            <cib>
                <configuration>
                    <resources>
                        <primitive id="CONFLICT"/>
                        <primitive id="A">
                            <meta_attributes>
                                <nvpair name="remote-node"
                                    value="GUEST_CONFLICT"
                                />
                            </meta_attributes>
                        </primitive>
                        <primitive id="B" class="ocf" provider="pacemaker"
                            type="remote"
                        >
                            <instance_attributes>
                                <nvpair name="server" value="REMOTE_CONFLICT"/>
                            </instance_attributes>
                        </primitive>
                        <primitive id="C">
                            <meta_attributes>
                                <nvpair name="remote-node" value="some"/>
                                <nvpair name="remote-addr"
                                    value="GUEST_ADDR_CONFLICT"
                                />
                            </meta_attributes>
                        </primitive>
                    </resources>
                </configuration>
            </cib>
        """)
        nodes = [
            NodeAddresses("RING0", "RING1", name="R1"),
            NodeAddresses("REMOTE_CONFLICT", name="B"),
            NodeAddresses("GUEST_CONFLICT", name="GUEST_CONFLICT"),
            NodeAddresses("GUEST_ADDR_CONFLICT", name="some"),
        ]
        return guest_node.validate_conflicts(tree, nodes, node_name, options)

    def assert_already_exists_error(
        self, conflict_name, node_name, options=None
    ):
        assert_report_item_list_equal(
            self.validate(node_name, options if options else {}),
            [
                (
                    severities.ERROR,
                    report_codes.ID_ALREADY_EXISTS,
                    {
                        "id": conflict_name,
                    },
                    None
                ),
            ]
        )


    def test_report_conflict_with_id(self):
        self.assert_already_exists_error("CONFLICT", "CONFLICT")

    def test_report_conflict_guest_node(self):
        self.assert_already_exists_error("GUEST_CONFLICT", "GUEST_CONFLICT")

    def test_report_conflict_guest_addr(self):
        self.assert_already_exists_error(
            "GUEST_ADDR_CONFLICT",
            "GUEST_ADDR_CONFLICT",
        )

    def test_report_conflict_guest_addr_by_addr(self):
        self.assert_already_exists_error(
            "GUEST_ADDR_CONFLICT",
            "GUEST_ADDR_CONFLICT",
        )

    def test_no_conflict_guest_node_whe_addr_is_different(self):
        self.assertEqual([], self.validate("GUEST_ADDR_CONFLICT", {
            "remote-addr": "different",
        }))

    def test_report_conflict_remote_node(self):
        self.assert_already_exists_error("REMOTE_CONFLICT", "REMOTE_CONFLICT")

    def test_no_conflict_remote_node_whe_addr_is_different(self):
        self.assertEqual([], self.validate("REMOTE_CONFLICT", {
            "remote-addr": "different",
        }))

    def test_report_conflict_remote_node_by_addr(self):
        self.assert_already_exists_error("REMOTE_CONFLICT", "different", {
            "remote-addr": "REMOTE_CONFLICT",
        })

class ValidateOptions(TestCase):
    def validate(self, options, name="some_name"):
        return guest_node.validate_set_as_guest(
            etree.fromstring('<cib/>'),
            [NodeAddresses(
                "EXISTING-HOST-RING0",
                "EXISTING-HOST-RING0",
                name="EXISTING-HOST-NAME"
            )],
            name,
            options
        )

    def test_no_report_on_valid(self):
        self.assertEqual(
            [],
            self.validate({}, "node1")
        )

    def test_report_invalid_option(self):
        assert_report_item_list_equal(
            self.validate({"invalid": "invalid"}, "node1"),
            [
                (
                    severities.ERROR,
                    report_codes.INVALID_OPTIONS,
                    {
                        "option_type": "guest",
                        "option_names": ["invalid"],
                        "allowed": sorted(guest_node.GUEST_OPTIONS),
                        "allowed_patterns": [],
                    },
                    None
                ),
            ]
        )

    def test_report_invalid_interval(self):
        assert_report_item_list_equal(
            self.validate({"remote-connect-timeout": "invalid"}, "node1"),
            [
                (
                    severities.ERROR,
                    report_codes.INVALID_OPTION_VALUE,
                    {
                        "option_name": "remote-connect-timeout",
                        "option_value": "invalid",
                    },
                    None
                ),
            ]
        )

    def test_report_invalid_node_name(self):
        assert_report_item_list_equal(
            self.validate({}, "EXISTING-HOST-NAME"),
            [
                (
                    severities.ERROR,
                    report_codes.ID_ALREADY_EXISTS,
                    {
                        "id": "EXISTING-HOST-NAME",
                    },
                    None
                ),
            ]
        )


class ValidateInNotGuest(TestCase):
    #guest_node.is_guest_node is tested here as well
    def test_no_report_on_non_guest(self):
        self.assertEqual(
            [],
            guest_node.validate_is_not_guest(etree.fromstring("<primitive/>"))
        )

    def test_report_when_is_guest(self):
        assert_report_item_list_equal(
            guest_node.validate_is_not_guest(etree.fromstring("""
                <primitive id="resource_id">
                    <meta_attributes>
                        <nvpair name="remote-node" value="node1" />
                    </meta_attributes>
                </primitive>
            """)),
            [
                (
                    severities.ERROR,
                    report_codes.RESOURCE_IS_GUEST_NODE_ALREADY,
                    {
                        "resource_id": "resource_id",
                    },
                    None
                ),
            ]
        )

class SetAsGuest(TestCase):
    def test_set_guest_meta_correctly(self):
        resource_element = etree.fromstring('<primitive id="A"/>')
        guest_node.set_as_guest(resource_element, "node1", connect_timeout="10")
        assert_xml_equal(
            etree.tostring(resource_element).decode(),
            """
                <primitive id="A">
                    <meta_attributes id="A-meta_attributes">
                        <nvpair id="A-meta_attributes-remote-connect-timeout"
                            name="remote-connect-timeout" value="10"
                        />
                        <nvpair id="A-meta_attributes-remote-node"
                            name="remote-node" value="node1"
                        />
                    </meta_attributes>
                </primitive>
            """
        )

class UnsetGuest(TestCase):
    def test_unset_all_guest_attributes(self):
        resource_element = etree.fromstring("""
            <primitive id="A">
                <meta_attributes id="B">
                    <nvpair id="C" name="remote-node" value="node1"/>
                    <nvpair id="D" name="remote-port" value="2222"/>
                    <nvpair id="E" name="remote-addr" value="node3"/>
                    <nvpair id="F" name="remote-connect-timeout" value="10"/>
                    <nvpair id="G" name="a" value="b"/>
                </meta_attributes>
            </primitive>
        """)
        guest_node.unset_guest(resource_element)
        assert_xml_equal(
            etree.tostring(resource_element).decode(),
            """
                <primitive id="A">
                    <meta_attributes id="B">
                        <nvpair id="G" name="a" value="b"/>
                    </meta_attributes>
                </primitive>
            """
        )

    def test_unset_all_guest_attributes_and_empty_meta_tag(self):
        resource_element = etree.fromstring("""
            <primitive id="A">
                <meta_attributes id="B">
                    <nvpair id="C" name="remote-node" value="node1"/>
                    <nvpair id="D" name="remote-port" value="2222"/>
                    <nvpair id="E" name="remote-addr" value="node3"/>
                    <nvpair id="F" name="remote-connect-timeout" value="10"/>
                </meta_attributes>
            </primitive>
        """)
        guest_node.unset_guest(resource_element)
        assert_xml_equal(
            etree.tostring(resource_element).decode(),
            '<primitive id="A"/>'
        )


class FindNodeList(TestCase, SetupPatchMixin):
    def assert_find_meta_attributes(self, xml, meta_attributes_xml_list):
        get_node = self.setup_patch("get_node", return_value=None)

        self.assertEquals(
            [None] * len(meta_attributes_xml_list),
            guest_node.find_node_list(etree.fromstring(xml))
        )

        for i, call in enumerate(get_node.mock_calls):
            assert_xml_equal(
                meta_attributes_xml_list[i],
                etree.tostring(call[1][0]).decode()
            )

    def test_get_no_nodes_when_no_primitives(self):
        self.assert_find_meta_attributes("<resources/>", [])

    def test_get_no_nodes_when_no_meta_remote_node(self):
        self.assert_find_meta_attributes(
            """
            <resources>
                <primitive>
                    <meta_attributes>
                        <nvpair name="remote-addr" value="G1"/>
                    </meta_attributes>
                </primitive>
            </resources>
            """,
            []
        )

    def test_get_multiple_nodes(self):
        self.assert_find_meta_attributes(
            """
            <resources>
                <primitive>
                    <meta_attributes>
                        <nvpair name="remote-node" value="G1"/>
                        <nvpair name="remote-addr" value="G1addr"/>
                    </meta_attributes>
                </primitive>
                <primitive>
                    <meta_attributes>
                        <nvpair name="remote-node" value="G2"/>
                    </meta_attributes>
                </primitive>
            </resources>
            """,
            [
                """
                <meta_attributes>
                    <nvpair name="remote-node" value="G1"/>
                    <nvpair name="remote-addr" value="G1addr"/>
                </meta_attributes>
                """,
                """
                <meta_attributes>
                    <nvpair name="remote-node" value="G2"/>
                </meta_attributes>
                """,
            ]
        )

class GetNode(TestCase):
    def assert_node(self, xml, expected_node):
        node = guest_node.get_node(etree.fromstring(xml))
        self.assertEquals(expected_node, (node.ring0, node.name))

    def test_return_none_when_is_not_guest_node(self):
        self.assertIsNone(guest_node.get_node(etree.fromstring(
            """
            <meta_attributes>
                <nvpair name="remote-addr" value="G1"/>
            </meta_attributes>
            """
        )))

    def test_return_same_host_and_name_when_remote_node_only(self):
        self.assert_node(
            """
            <meta_attributes>
                <nvpair name="remote-node" value="G1"/>
            </meta_attributes>
            """,
            ("G1", "G1")
        )

    def test_return_different_host_and_name_when_remote_addr_there(self):
        self.assert_node(
            """
            <meta_attributes>
                <nvpair name="remote-node" value="G1"/>
                <nvpair name="remote-addr" value="G1addr"/>
            </meta_attributes>
            """,
            ("G1addr", "G1")
        )

class GetHost(TestCase):
    def assert_find_host(self, host, xml):
        self.assertEqual(host, guest_node.get_host(etree.fromstring(xml)))

    def test_return_host_from_remote_addr(self):
        self.assert_find_host("HOST", """
            <primitive>
                <meta_attributes>
                    <nvpair name="remote-node" value="NODE" />
                    <nvpair name="remote-addr" value="HOST" />
                </meta_attributes>
            </primitive>
        """)

    def test_return_host_from_remote_node(self):
        self.assert_find_host("HOST", """
            <primitive>
                <meta_attributes>
                    <nvpair name="remote-node" value="HOST" />
                </meta_attributes>
            </primitive>
        """)

    def test_return_none(self):
        self.assert_find_host(None, """
            <primitive>
                <meta_attributes>
                    <nvpair name="any" value="HOST" />
                </meta_attributes>
            </primitive>
        """)

class FindNodeResources(TestCase):
    def assert_return_resources(self, identifier):
        resources_section = etree.fromstring("""
            <resources>
                <primitive id="RESOURCE_ID">
                    <meta_attributes>
                        <nvpair name="remote-node" value="NODE_NAME" />
                        <nvpair name="remote-addr" value="NODE_HOST" />
                    </meta_attributes>
                </primitive>
            </resources>
        """)
        self.assertEquals(
            "RESOURCE_ID",
            guest_node.find_node_resources(resources_section, identifier)[0]
                .attrib["id"]
        )

    def test_return_resources_by_resource_id(self):
        self.assert_return_resources("RESOURCE_ID")

    def test_return_resources_by_node_name(self):
        self.assert_return_resources("NODE_NAME")

    def test_return_resources_by_node_host(self):
        self.assert_return_resources("NODE_HOST")

    def test_no_result_when_no_guest_nodes(self):
        resources_section = etree.fromstring(
            '<resources><primitive id="RESOURCE_ID"/></resources>'
        )
        self.assertEquals([], guest_node.find_node_resources(
            resources_section,
            "RESOURCE_ID"
        ))



from __future__ import (
    absolute_import,
    division,
    print_function,
)

from pcs.test.tools.pcs_unittest import TestCase

from pcs.common import report_codes
from pcs.lib.booth import env
from pcs.lib.errors import ReportItemSeverity as severities
from pcs.test.tools.assertions import assert_raise_library_error
from pcs.test.tools.misc import create_patcher
from pcs.test.tools.pcs_unittest import mock

patch_env = create_patcher("pcs.lib.booth.env")

class GetConfigFileNameTest(TestCase):
    @patch_env("os.path.exists")
    def test_refuse_when_name_starts_with_slash(self, mock_path_exists):
        mock_path_exists.return_value = True
        assert_raise_library_error(
            lambda: env.get_config_file_name("/booth"),
            (
                severities.ERROR,
                report_codes.BOOTH_INVALID_NAME,
                {
                    "name": "/booth",
                    "reason": "contains illegal character '/'",
                }
            ),
        )

class BoothEnvTest(TestCase):
    @patch_env("RealFile")
    def test_get_content_from_file(self, mock_real_file):
        mock_real_file.return_value = mock.MagicMock(
            read=mock.MagicMock(return_value="content")
        )
        self.assertEqual(
            "content",
            env.BoothEnv("report processor", env_data={"name": "booth"})
                .get_config_content()
        )

    @patch_env("set_keyfile_access")
    @patch_env("RealFile")
    def test_create_config(self, mock_real_file, mock_set_keyfile_access):
        mock_file = mock.MagicMock(
            assert_no_conflict_with_existing=mock.MagicMock(),
            write=mock.MagicMock(),
        )
        mock_real_file.return_value = mock_file


        env.BoothEnv(
            "report processor",
            env_data={"name": "booth"}
        ).create_config("a", can_overwrite_existing=True)

        self.assertEqual(mock_file.assert_no_conflict_with_existing.mock_calls,[
            mock.call('report processor', True),
        ])
        self.assertEqual(mock_file.write.mock_calls, [mock.call('a')])

    @patch_env("RealFile")
    def test_push_config(self, mock_real_file):
        mock_file = mock.MagicMock(
            assert_no_conflict_with_existing=mock.MagicMock(),
            write=mock.MagicMock(),
        )
        mock_real_file.return_value = mock_file
        env.BoothEnv(
            "report processor",
            env_data={"name": "booth"}
        ).push_config("a")
        mock_file.write.assert_called_once_with("a")



    def test_export_config_file_when_was_present_in_env_data(self):
        self.assertEqual(
            env.BoothEnv(
                "report processor",
                {
                    "name": "booth-name",
                    "config_file": {
                        "content": "a\nb",
                    },
                    "key_file": {
                        "content": "secure",
                    },
                    "key_path": "/path/to/file.key",
                }
            ).export(),
            {
                "config_file": {
                    "content": "a\nb",
                    "can_overwrite_existing_file": False,
                    "no_existing_file_expected": False,
                    "is_binary": False,
                },
                "key_file": {
                    "content": "secure",
                    "can_overwrite_existing_file": False,
                    "no_existing_file_expected": False,
                    "is_binary": True,
                },
            }
        )

    def test_do_not_export_config_file_when_no_provided(self):
        self.assertEqual(
            env.BoothEnv("report processor", {"name": "booth"}).export(),
            {}
        )

class SetKeyfileAccessTest(TestCase):
    @patch_env("os.chmod")
    @patch_env("os.chown")
    @patch_env("grp.getgrnam")
    @patch_env("pwd.getpwnam")
    @patch_env("settings")
    def test_do_everything_to_set_desired_file_access(
        self, settings, getpwnam, getgrnam, chown, chmod
    ):
        file_path = "/tmp/some_booth_file"
        env.set_keyfile_access(file_path)

        getpwnam.assert_called_once_with(settings.pacemaker_uname)
        getgrnam.assert_called_once_with(settings.pacemaker_gname)

        chown.assert_called_once_with(
            file_path,
            getpwnam.return_value.pw_uid,
            getgrnam.return_value.gr_gid,
        )

    @patch_env("pwd.getpwnam", mock.MagicMock(side_effect=KeyError))
    @patch_env("settings.pacemaker_uname", "some-user")
    def test_raises_when_cannot_get_uid(self):
        assert_raise_library_error(
            lambda: env.set_keyfile_access("/booth"),
            (
                severities.ERROR,
                report_codes.UNABLE_TO_DETERMINE_USER_UID,
                {
                    "user": "some-user",
                }
            ),
        )

    @patch_env("grp.getgrnam", mock.MagicMock(side_effect=KeyError))
    @patch_env("pwd.getpwnam", mock.MagicMock())
    @patch_env("settings.pacemaker_gname", "some-group")
    def test_raises_when_cannot_get_gid(self):
        assert_raise_library_error(
            lambda: env.set_keyfile_access("/booth"),
            (
                severities.ERROR,
                report_codes.UNABLE_TO_DETERMINE_GROUP_GID,
                {
                    "group": "some-group",
                }
            ),
        )

    @patch_env("format_environment_error", mock.Mock(return_value="err"))
    @patch_env("os.chown", mock.MagicMock(side_effect=EnvironmentError()))
    @patch_env("grp.getgrnam", mock.MagicMock())
    @patch_env("pwd.getpwnam", mock.MagicMock())
    @patch_env("settings.pacemaker_gname", "some-group")
    def test_raises_when_cannot_chown(self):
        assert_raise_library_error(
            lambda: env.set_keyfile_access("/booth"),
            (
                severities.ERROR,
                report_codes.FILE_IO_ERROR,
                {
                    'reason': 'err',
                    'file_role': u'BOOTH_KEY',
                    'file_path': '/booth',
                    'operation': u'chown',
                }
            ),
        )

    @patch_env("format_environment_error", mock.Mock(return_value="err"))
    @patch_env("os.chmod", mock.MagicMock(side_effect=EnvironmentError()))
    @patch_env("os.chown", mock.MagicMock())
    @patch_env("grp.getgrnam", mock.MagicMock())
    @patch_env("pwd.getpwnam", mock.MagicMock())
    @patch_env("settings.pacemaker_gname", "some-group")
    def test_raises_when_cannot_chmod(self):
        assert_raise_library_error(
            lambda: env.set_keyfile_access("/booth"),
            (
                severities.ERROR,
                report_codes.FILE_IO_ERROR,
                {
                    'reason': 'err',
                    'file_role': u'BOOTH_KEY',
                    'file_path': '/booth',
                    'operation': u'chmod',
                }
            ),
        )



from __future__ import (
    absolute_import,
    division,
    print_function,
)

from pcs.common import env_file_role_codes
from pcs.lib.env_file import RealFile
from pcs import settings


class PacemakerEnv(object):
    def __init__(self):
        """
        callable get_cib should return cib as lxml tree
        """
        self.__authkey = RealFile(
            file_role=env_file_role_codes.PACEMAKER_AUTHKEY,
            file_path=settings.pacemaker_authkey_file,
            is_binary=True,
        )

    @property
    def has_authkey(self):
        return self.__authkey.exists

    def get_authkey_content(self):
        return self.__authkey.read()
